### Faster ≠ smarter: children with higher levels of ability take longer to give incorrect answers, especially when the task matches their ability
##  The analytic script
#   The data and other supplementary materials can be accessed via: https://osf.io/dhu7f/

### This script allows the users to replicate:
# (1) basic data cleaning and sample description, 
# (2) preliminary IRT estimates, 
# (3) data preparation for main analyses, and 
# (4) the main analyses (comparison of multilevel regressions with IRT estimates)

# Plots can be reproduced using the plot_script.R in the same repository (https://osf.io/dhu7f/), once all the code in this script has been run. 
# The a posteriori sensitivity analysis can be re-run using the sensivity_analysis_script.R in the same repository (https://osf.io/dhu7f/) anytime


#### PACKAGES AND DATA ####

## working directory
setwd("...") # Insert your working Directory


## packages
# install (if necessary)
list.of.packages <- c("psych", "reshape2", "mirt", "lme4", "lmerTest", "car", "performance")
new.packages <- list.of.packages[!(list.of.packages %in% installed.packages()[,"Package"])]
if(length(new.packages)) install.packages(new.packages)
print(paste(length(not_installed), "packages had to be installed."))

# load
library(psych)
library(reshape2)
library(mirt)
library(lme4)
library(lmerTest)
library(car)
library(performance)

## load data
triton.data <- read.csv("Triton_data.csv", encoding = "UTF-8")


#### (1) DESCRIPTIVE STATICTICS ####


## sample size
sample <- nrow(triton.data)


## gender descriptives (0 = girls, 1 = boys)
gender.abs <- table(triton.data$gender) # frequencies
gender.rel <- proportions(gender.abs) # proportions


## age descriptives
age.mean <- mean(triton.data$age_abs, na.rm = TRUE) # mean
age.sd <- sd(triton.data$age_abs, na.rm = TRUE) # standard deviation
age.min <- triton.data[which.min(triton.data$age_abs), c("age_year", "age_month")] # minimal age
age.max <- triton.data[which.max(triton.data$age_abs), c("age_year", "age_month")] # maximum age


## OPTIONAL :: grade and school descriptives
# grade.abs <- table(triton.data$grade) # distribution of grades - frequencies
# grade.rel <- proportions(grade.abs) # distribution of grades - proportions
# school.abs <- table(triton.data$school_id) # distribution of school samples - frequencies



#### (2) PRELIMINARY IRT MODELS ####

## calculate the guessing parameter of items based on number of possible solutions
guess.fixed <- 1 / c(5, 5, 15, 15, 20, 15, 5, 10, 5, 20, 5, 5, 5, 20, 5, 
                     60, 5, 5, 20, 5, 5, 15, 5, 10, 20, 5, 5, 20, 60)


## Rasch model estimation
triton.rasch <- mirt(data = triton.data[ , paste0("item_correct_", 1:29)], 
                     model = 1, itemtype = "Rasch", SE = TRUE, guess = 0, 
                     technical = list(NCYCLES = 1000), TOL = 1e-7)


## Quasi-Rasch model estimation
triton.quasi.rasch <- mirt(data = triton.data[ , paste0("item_correct_", 1:29)], 

                           model = 1, itemtype = "Rasch", SE = TRUE, guess = guess.fixed,

                           technical = list(NCYCLES = 1000), TOL = 1e-7)


## extract model fit indicies
# M2, df, p, RMSEA, SRMSR, and TLI statistics
M2(triton.rasch, na.rm = TRUE) # Rasch model
M2(triton.quasi.rasch, na.rm = TRUE) # Quasi-Rasch model

# AIC and BIC
anova(triton.rasch, triton.quasi.rasch)


## extract model parameters
# item difficulty (b) and guessing parameter (g)
coef(triton.rasch, IRTpars = TRUE, simplify = TRUE) # Rasch model
coef(triton.quasi.rasch, IRTpars = TRUE, simplify = TRUE) # Quasi-Rasch model

# extract empirical reliability
fscores(triton.rasch, returnER = TRUE) # Rasch model
fscores(triton.quasi.rasch, returnER = TRUE) # Quasi-Rasch model

# OPTIONAL :: plot local reliability
# plot(triton.rasch, type="rxx", MI = 1000)
# plot(triton.quasi.rasch, type="rxx", MI = 1000)

# item characteristic curves (ICCs)
plot(triton.rasch, type = "trace")
plot(triton.quasi.rasch, type = "trace")


#### (2.5) ITEM ANALYSIS AND ABILITY ESTIMATES ####

## item analysis
# descriptives of item correctness
item.correct.descr <- describe(triton.data[, paste0("item_correct_", 1:29)])

# descriptives of item response time
item.time.descr <- describe(triton.data[, paste0("item_time_", 1:29)])

# item parameters from Quasi-Rasch model 
triton.irt.pars <- coef(triton.quasi.rasch, IRTpars = TRUE, simplify = TRUE)

# merging to table for supplementary materials
triton.item.analysis <- data.frame(item.correct.descr[, c("vars", "n", "mean", "sd")],
                                   item.time.descr[, c("mean", "sd", "median")], 
                                   triton.irt.pars$items[ , c("b", "g")])
colnames(triton.item.analysis) <- c("item", "n", "mean_correct", "sd_correct", 
                                    "mean_time", "sd_time", "median_time", "difficulty", "guessing")


## ability estimates (theta) from Quasi-Rasch model 
triton.theta <- data.frame(triton.data$id, fscores(triton.quasi.rasch))
colnames(triton.theta) <- c("id", "ability")



#### (3) DATA TRANSFORMATION FOR MAIN ANALYSES ####

## create long data for item correctness
correct.long <- melt(triton.data, id.vars = "id", measure.vars = paste0("item_correct_", 1:29),
                     variable.name = "item", value.name = "correct")

# transform item variable to numeric
correct.long$item <- as.numeric(as.factor(correct.long$item))


## create long dataset for item response time
time.long <- melt(triton.data, id.vars = "id", measure.vars = paste0("item_time_", 1:29),
                  variable.name = "item", value.name = "time")
# transform item variable to numeric
time.long$item <- as.numeric(as.factor(time.long$item))


## merge the long data to create to the final dataset
triton.data.long <- merge(correct.long, time.long, by = c("id", "item"))

# assign item difficulty to every item
triton.data.long <- merge(triton.data.long, triton.item.analysis[, c("item", "difficulty")],
                          by = "item")

# assign ability to every participant
triton.data.long <- merge(triton.data.long, triton.theta, by = "id")

# compute ability-difficulty distance for every person and item
triton.data.long$distance <- triton.data.long$ability - triton.data.long$difficulty

# copute an absolute value of that discance
triton.data.long$distance_abs <- abs(triton.data.long$distance)

# sort rows by id and items
triton.data.long <- triton.data.long[order(triton.data.long$id, triton.data.long$item), ]
rownames(triton.data.long) <- 1:nrow(triton.data.long)



#### (4) MAIN ANALYSES ####

## Establish the null model
model.0 <- lmer(log(time) ~ 1 + (1 | id) + (1 | item), data = triton.data.long) # intercept

# model coefficients
summary(model.0)
# test of significance for random effects
rand(model.0)
# confidence intervals of parameters 
confint(model.0)
  # note: random effects are in std. deviations squared to obtain variance
# R-squared statistic
r2(model.0)


## Model A1 (F > C phenomenon only)
model.a1 <- lmer(log(time) ~ correct + (1 | id) + (1 | item), data = triton.data.long)

# model coefficients
summary(model.a1)
# test of significance for random effects
rand(model.a1)
# confidence intervals of parameters 
confint(model.a1)
  # note: random effects are in std. deviations squared to obtain variance
# R-squared statistic
r2(model.a1)


## Model A2 (controlling for difficulty and ability)
model.a2 <- lmer(log(time) ~ correct + difficulty + ability + (1 | id) + (1 | item), 
                 data = triton.data.long)

# model coefficients
summary(model.a2)
# multicollinearity check
vif(model.a2)
# test of significance for random effects
rand(model.a2)
# confidence intervals of parameters 
confint(model.a2)
  # note: random effects are in std. deviations, square to obtain variance
# R-squared statistic
r2(model.a2)


## Model A3 (F > C and ability interaction)
model.a3 <- lmer(log(time) ~ correct + difficulty + ability + ability:correct + 
                   (1 | id) + (1 | item), data = triton.data.long)

# model coefficients
summary(model.a3)
# multicollinearity check
vif(model.a3)
# test of significance for random effects
rand(model.a3)
# confidence intervals of parameters 
confint(model.a3)
  # note: random effects are in std. deviations, square to obtain variance
# R-squared statistic
r2(model.a3)


## Exploratory model A4 (difficulty and ability interaction)
model.a4 <- lmer(log(time) ~ correct + difficulty + ability + ability:correct + difficulty:ability + 
                       (1 | id) + (1 | item), data = triton.data.long)

# model coefficients
summary(model.a4)
# multicollinearity check
vif(model.a4)
# test of significance for random effects
rand(model.a4)
# confidence intervals of parameters 
confint(model.a4)
  # note: random effects are in std. deviations, square to obtain variance
# R-squared statistic
r2(model.a4)


## Comparison of 0, A1, A2, A3, and A4 models
anova(model.0, model.a1, model.a2, model.a3, model.a4)



## Model B1 (Thissen's model)
model.b1 <- lmer(log(time) ~ distance_abs + (1 | id) + (1 | item), data = triton.data.long)

# model coefficients
summary(model.b1)
# test of significance for random effects
rand(model.b1)
# confidence intervals of parameters 
confint(model.b1)
  # note: random effects are in std. deviations, square to obtain variance
# R-squared statistic
r2(model.b1)


## Model B2 (controlling for item correctness)
model.b2 <- lmer(log(time) ~ distance_abs + correct + (1 | id) + (1 | item), 
                 data = triton.data.long)

# model coefficients
summary(model.b2)
# multicollinearity check
vif(model.b2)
# test of significance for random effects
rand(model.b2)
# confidence intervals of parameters 
confint(model.b2)
  # note: random effects are in std. deviations squared to obtain variance
# R-squared statistic
r2(model.b2)


## Model B3 (ability-difficulty distance and item correctness interaction)
model.b3 <- lmer(log(time) ~ distance_abs + correct + distance_abs:correct + 
                   (1 | id) + (1 | item), data = triton.data.long)

# model coefficients
summary(model.b3)
# multicollinearity check
vif(model.b3)
# test of significance for random effects
rand(model.b3)
# confidence intervals of parameters 
confint(model.b3)
  # note: random effects are in standard deviations, square to obtain variance
# R-squared statistic
r2(model.b3)


## Comparison of 0, B1, B2, and B3 models
anova(model.0, model.b1, model.b2, model.b3)
